/**************************************************************************
** FILE NAME     : IFX_Errors.h
** PROJECT       : TR69
** MODULES       : List of all possible error codes return by TR69
** SRC VERSION   : V2.0
** DATE          : 15-12-2004
** AUTHOR        : Narendra.V.P
** DESCRIPTION   :
** REFERENCE     : Coding guide lines.
** COPYRIGHT     : Copyright (c) 2006
**                 Infineon Technologies AG,
**                 Am Campeon 1-12, 85579 Neubiberg, Germany
**
** Any use of this software is subject to the conclusion of a respective
** License agreement. Without such a License agreement no rights to the
** software are granted
**
** HISTORY       :
** $Date       $Author      $Comment
***********************************************************************/

/*! \file ifx_error.h
    \brief This file defines the errors for IFX MAPI.	
*/


#ifndef __IFIN_TR69_ERRORS_H__
#define __IFIN_TR69_ERRORS_H__



/*! \enum e_IFIN_TR69_Ecode
    \brief Enum containing list of all possible errors types .
*/
typedef enum{
  /* Transaction User Layer Errors */
  IFIN_TR69_NO_ERROR = 1,   /*!< TODO  */
  IFIN_TR69_NO_DIALOG,
  IFIN_TR69_TU_MEMORY_ERROR,
  IFIN_TR69_TU_TIMER_ERROR,
  IFIN_TR69_TU_TRANSPORT_ERROR,
  /* Transport Layer Errors */
  IFIN_TR69_TRERR_OK = 6,      /*!< TODO  */
  IFIN_TR69_TRERR_HOSTADDRFAIL,
  IFIN_TR69_TRERR_MEM,
  IFIN_TR69_TRERR_SOCKERR,
  IFIN_TR69_TRERR_TIMEOUT,
  IFIN_TR69_TRERR_CONNREFUSED,
  IFIN_TR69_TRERR_HOSTERR,
  IFIN_TR69_TRERR_NETWORKERR,
  IFIN_TR69_TRERR_UNSUPPORTEDPROTOCOL,
  IFIN_TR69_TRERR_LISTOVERFLOW,
  IFIN_TR69_TRERR_BINDFAILED,
  IFIN_TR69_TRERR_CONNRESET,
  IFIN_TR69_TRERR_SENDFAILED,
  IFIN_TR69_TRERR_CONN_PENDING,
  IFIN_TR69_TRERR_NOCONN,
  IFIN_TR69_TRERR_SOCK,
  IFIN_TR69_TRERR_LISTEN_FAILED,
  IFIN_TR69_TRERR_CONNFAILED,
  IFIN_TR69_TRERR_CONNAGAIN,
  IFIN_TR69_TRERR_PORTERR,
  IFIN_TR69_TRERR_CONN_CLOSED,
  /* Transaction Layer Errors */
  IFIN_TR69_TLERR_OK = 28,          /*!< TODO  */
  IFIN_TR69_TLERR_WRONG_APIID,
  IFIN_TR69_TLERR_NOTCB,
  IFIN_TR69_TLERR_WRONGMSG,
  IFIN_TR69_TLERR_IGNOREEVENT,
  IFIN_TR69_TLERR_MEM,
  IFIN_TR69_TLERR_TIMER,
  IFIN_TR69_TLERR_ENCERR,
  IFIN_TR69_TLERR_TRANSPORTERR,
  /* Decoder Errors */
  /* Duplicate mandatory headers */
  IFIN_TR69_DUP_CSEQ = 50, /*!< Raised when a duplicate CSEQ hdr is found  */
  /* Raised when a duplicate CSEQ hdr is found */
  IFIN_TR69_DUP_CALLID, /*!< Raised when a duplicate CALL ID hdr is found  */ 
  /* Raised when a duplicate CALL ID hdr is found */
  IFIN_TR69_DUP_TO,  /*!< Raised when a duplicate TO hdr is found  */ 
  /* Raised when a duplicate TO hdr is found */
  IFIN_TR69_DUP_FROM,  /*!<  Raised when a duplicate FROM hdr is found  */ 
  /* Raised when a duplicate FROM hdr is found */
  IFIN_TR69_DUP_MAXFOR,  /*!< Raised when a duplicate Max Forwards hdr is found  */  
  /* Raised when a duplicate Max Forwards hdr is found */
  /* Insufficient SDP length */
  IFIN_TR69_INSUFF_SDP,  /*!< Insufficient SDP length  */  
  /* Invalid Message format */
  IFIN_TR69_INVALID_MSG,  /*!< Invalid Message format  */  
  /* Mandatory Headers Missing */
  IFIN_TR69_MANHDR_MISSING,  /*!< Mandatory Headers Missing */  
  /* Different Method in Req Line and CSeq */
  IFIN_TR69_METHOD_MISMATCH,  /*!< Different Method in Req Line and CSeq */  
  /* Invalid Status Code */
  IFIN_TR69_STATUSCODE_INVALID,  /*!< Invalid Status Code  */  
  /* Errors in Fmt of mandatory Hdrs */
  IFIN_TR69_VIAPARAM_FORMAT_ERR, /*!< Errors in Fmt of mandatory Hdrs  */  /* 60 */
  IFIN_TR69_INVALID_CALLID,  /*!< Invalid call ID  */
  /*Maximum Limit of string or array exceeded*/
  IFIN_TR69_CALLID_EXCEEDED,
  IFIN_TR69_METHOD_EXCEEDED,
  IFIN_TR69_CSEQVAL_EXCEEDED,
  IFIN_TR69_VERSION_EXCEEDED,
  IFIN_TR69_REASONPHRASE_EXCEEDED,
  IFIN_TR69_DISPLAYNAME_EXCEEDED,
  IFIN_TR69_MAXFORWARD_EXCEEDED,
  IFIN_TR69_HOSTNAME_EXCEEDED,
  IFIN_TR69_HOSTPORT_EXCEEDED,
  IFIN_TR69_USERINFO_EXCEEDED,
  IFIN_TR69_TAGPARAM_EXCEEDED,
  IFIN_TR69_GENPARMTOK_EXCEEDED,
  IFIN_TR69_OTHERPARM_EXCEEDED,
  IFIN_TR69_TTLPARM_EXCEEDED, /* 75 */
  IFIN_TR69_HDR_EXCEEDED,
  IFIN_TR69_SUBJECT_EXCEEDED,
  IFIN_TR69_EXPIRES_EXCEEDED,
  IFIN_TR69_ORGANIZATION_EXCEEDED,
  IFIN_TR69_BRANCHPARM_EXCEEDED,
  IFIN_TR69_MADDRPARM_EXCEEDED,
  IFIN_TR69_PROTO_EXCEEDED,
  IFIN_TR69_VIATRANSP_EXCEEDED,
  IFIN_TR69_INREPTOCALLID_EXCEEDED,
  IFIN_TR69_MINEXPIRES_EXCEEDED,
  IFIN_TR69_TOKEN_EXCEEDED,
  IFIN_TR69_PROXYREQUIRE_EXCEEDED,
  IFIN_TR69_CONTENC_EXCEEDED,
  IFIN_TR69_MPARAM_EXCEEDED,
  IFIN_TR69_MEDIASUBTYPE_EXCEEDED,     /* 90 */
  IFIN_TR69_DISCEXT_EXCEEDED,
  IFIN_TR69_COMMENT_EXCEEDED,
  IFIN_TR69_PRODUCT_EXCEEDED,
  IFIN_TR69_ABSURISCHEME_EXCEEDED,
  IFIN_TR69_OPAQUE_EXCEEDED,
  IFIN_TR69_SEGMENT_EXCEEDED,
  IFIN_TR69_ABSURIQUERY_EXCEEDED,
  IFIN_TR69_ABSURIAUTH_EXCEEDED,
  IFIN_TR69_PURPOSETOKEN_EXCEEDED,
  IFIN_TR69_MAX_RPLYTOPARAM_EXCEEDED, /* 100 */
  IFIN_TR69_MAX_RFRTOPARAM_EXCEEDED,
  IFIN_TR69_MAX_RFRBYPARAM_EXCEEDED,
  IFIN_TR69_MAX_ERRORURI_EXCEEDED,
  IFIN_TR69_MAX_INFO_EXCEEDED,
  IFIN_TR69_MAX_ALERTPARAM_EXCEEDED,
  IFIN_TR69_MAX_ACCEPTRANGE_EXCEEDED,
  IFIN_TR69_MAX_OPTTAG_EXCEEDED,
  IFIN_TR69_MAX_ENCODING_EXCEEDED,
  IFIN_TR69_MAXENC_EXCEEDED,
  IFIN_TR69_MAX_LANGUAGE_EXCEEDED,     /* 110 */
  IFIN_TR69_NONCECOUNT_EXCEEDED,
  IFIN_TR69_MAX_AINFO_EXCEEDED,
  IFIN_TR69_REQUIRE_EXCEEDED,
  IFIN_TR69_MAXSEG_EXCEEDED,
  IFIN_TR69_MAXURI_EXCEEDED,
  IFIN_TR69_MAXDIGRESP_EXCEEDED,
  IFIN_TR69_MAXMPARAM_EXCEEDED,
  IFIN_TR69_MAXVIAPARAM_EXCEEDED,
  IFIN_TR69_MAXRRPARAM_EXCEEDED,
  IFIN_TR69_MAXCONTACTPARAM_EXCEEDED,  /* 120 */
  IFIN_TR69_MAX_ACCEPTPARAM_EXCEEDED,
  IFIN_TR69_MAXGENPARAM_EXCEEDED,
  IFIN_TR69_MAXINFOPARAM_EXCEEDED,
  IFIN_TR69_MAXCONTPARAM_EXCEEDED,
  IFIN_TR69_MAXCALLID_EXCEEDED, /* 125 */
  IFIN_TR69_MAXROUTE_EXCEEDED,
  IFIN_TR69_MAXRECROUTE_EXCEEDED,
  IFIN_TR69_MAXOPTTAG_EXCEEDED,
  IFIN_TR69_MAXSERVERVAL_EXCEEDED,
  IFIN_TR69_MAXMETHOD_EXCEEDED,    /* 130 */
  IFIN_TR69_TAGPARM_EXCEEDED,
  IFIN_TR69_MAX_FROMPARAM_EXCEEDED,
  IFIN_TR69_OPTTAG_EXCEEDED,
  IFIN_TR69_MAX_LANGTAG_EXCEEDED,
  IFIN_TR69_MAXDOMAIN_EXCEEDED,
  IFIN_TR69_MAXFUTEXT_EXCEEDED,
  IFIN_TR69_MAXSERV_EXCEEDED,
  IFIN_TR69_MAXAREASPEC_EXCEEDED,
  IFIN_TR69_MAXPHONDIG_EXCEEDED,
  IFIN_TR69_MAXTOKEN_EXCEEDED,     /* 140 */
  IFIN_TR69_MAX_CHAL_EXCEEDED,
  IFIN_TR69_AUTHPARAMNAME_EXCEEDED,
  IFIN_TR69_MAXDIG_EXCEEDED,
  IFIN_TR69_MAX_QOP_EXCEEDED,
  IFIN_TR69_MAXWARNVAL_EXCEEDED,
  IFIN_TR69_MAXCHAL_EXCEEDED,
  IFIN_TR69_MAXCRED_EXCEEDED,
  IFIN_TR69_MAX_REPLCPARAM_EXCEEDED,
  IFIN_TR69_MAX_SUBSTATE_EXCEEDED,
  IFIN_TR69_MAX_EVENTRSN_EXCEEDED, /* 150 */
  IFIN_TR69_MAX_EVNTTMPLT_EXCEEDED,
  /* Other errors Like in format of msg etc. */
  IFIN_TR69_OUTOFMEM,             /*!< TODO  */
  IFIN_TR69_INVALID_MADDR,
  IFIN_TR69_INVALID_HDR,
  IFIN_TR69_INVALID_UTF8TRIM,
  IFIN_TR69_INVALID_METHOD,
  IFIN_TR69_INVALID_DIGIT,
  IFIN_TR69_INVALID_TOKEN,
  IFIN_TR69_INVALID_PARAMCHAR,
  IFIN_TR69_INVALID_WORD,  /* 160 */
  IFIN_TR69_INVALID_URIC,
  IFIN_TR69_INVALID_PCHAR,
  IFIN_TR69_INVALID_SCHEME,
  IFIN_TR69_INVALID_QUOTEDSTRING,
  IFIN_TR69_INVALID_RESPDIGEST,
  IFIN_TR69_INVALID_REGNAME,
  IFIN_TR69_INVALID_LANGUAGE,
  IFIN_TR69_HEADERINFO_INVALID,
  IFIN_TR69_INVALID_LWS_IN_PROXY_AUTHENTICATE,
  IFIN_TR69_INVALID_PROXY_AUTHENTICATE, /* 170 */
  IFIN_TR69_INVALID_DIGEST_CLN,
  IFIN_TR69_INVALID_PROXY_AUTHORIZATION,
  IFIN_TR69_INVALID_DIGEST_RESPONSE,
  IFIN_TR69_INVALID_NAMEADDR_FORMAT,
  IFIN_TR69_CONTACTPARAMFORMAT_INVALID,
  IFIN_TR69_QVALFORMAT_INVALID,
  IFIN_TR69_INVALID_AUTHORIZATION,
  IFIN_TR69_MPARAMFMT_INVALID,
  IFIN_TR69_MEDIATYPEFMT_INVALID,
  IFIN_TR69_INVALID_COMMENT,       /* 180 */
  IFIN_TR69_PRODUCT_INVALID,
  IFIN_TR69_INVALID_WWW_AUTHENTICATE,
  IFIN_TR69_MEDIARANGEFMT_INVALID,
  IFIN_TR69_ABSURIFMT_INVALID,     /* 184 */
  IFIN_TR69_INVALID_FMT,
  IFIN_TR69_DECODE_END_POINTER,
  IFIN_TR69_DECODE_CODE_MORE_DIGIT,
  IFIN_TR69_DECODE_WRONG_WARN_AGENT,
  IFIN_TR69_DECODE_WRONG_DATE_FORMAT,
  IFIN_TR69_DECODE_NOT_A_DIGIT,    /* 190 */
  IFIN_TR69_DECODE_LENGTH_GREATER_THAN_REQUIRED,
  IFIN_TR69_DECODE_TIME_STAMP_NOT_SUFFICIENT_INFO,
  IFIN_TR69_VERSION_MISMATCH,
  IFIN_TR69_DIGCLNFMT_INVALID,
  IFIN_TR69_INVALID_LOCALPHONENOFMT,
  IFIN_TR69_INVALID_ERRURIFMT,
  IFIN_TR69_INVALID_AUTHPARAMFMT,
  IFIN_TR69_INVALID_ADDR,
  IFIN_TR69_INVALID_PHONEDIG,
  IFIN_TR69_ACCEPTLANGFMT_INVALID,  /* 200 */
  IFIN_TR69_INVALID_NC,
  IFIN_TR69_INVALID_TIMESTAMP,
  IFIN_TR69_INVALID_DIGCLNFMT,
  IFIN_TR69_INVALID_HOSTINFO,
  IFIN_TR69_INVALID_MSGFMT,
  IFIN_TR69_INVALID_STATCODE,
  IFIN_TR69_USERINFO_ABSENT,
  IFIN_TR69_INVALID_RECROUTE,
  IFIN_TR69_INVALID_REASONPHRASE,
  IFIN_TR69_INVALID_HDRVALUE,      /* 210 */
  IFIN_TR69_INVALID_PASSWD,
  IFIN_TR69_INVALID_TAGPARAM,
  IFIN_TR69_INVALID_DISPLAYNAME,
  IFIN_TR69_INVALID_GENVALUE,
  IFIN_TR69_MAX_RETRYPARAM_EXCEEDED,
  IFIN_TR69_DUPHDR,
  IFIN_TR69_INVALID_CONTENTLEN,
  IFIN_TR69_INVALID_ROUTE,
  IFIN_TR69_INVALID_AUTHSCHEME,
  IFIN_TR69_RPORT_EXCEEDED,        /* 220 */
  /* SDP Errors */
  IFIN_SDP_INVALID_PROTO_VER = 250,     /*!< TODO  */
  IFIN_SDP_INVALID_ORIGIN,
  IFIN_SDP_INVALID_SESSION,
  IFIN_SDP_INVALID_OPTIONAL_FIELD,
  IFIN_SDP_INVALID_SEQ,
  IFIN_SDP_NO_CONN_FIELD,
  IFIN_SDP_NO_TIME_FIELD,
  IFIN_SDP_INCOMPLETE_MSG,
  IFIN_SDP_NO_CRLF,
  IFIN_SDP_INVALID_OPTIONAL_STRING,
  /* Encoder Errors */
  IFIN_TR69_ENCSUCCESS = 300,            /*!< TODO  */
  /* 0 */
  IFIN_TR69_ENCERR_USERPARAMTYPE,         /*!< TODO  */
  IFIN_TR69_ENCERR_TRANSTYPE,
  IFIN_TR69_ENCERR_SERVERVALTYPE,
  IFIN_TR69_ENCERR_RETRYPARAMTYPE,
  IFIN_TR69_ENCERR_QOPVALTYPE,
  /* 5 */
  IFIN_TR69_ENCERR_ALGTYPE,              /*!< TODO  */
  IFIN_TR69_ENCERR_TR69URITYPE,
  IFIN_TR69_ENCERR_BOOLEAN,
  IFIN_TR69_ENCERR_DIGESTCLNTYPE,
  IFIN_TR69_ENCERR_CHALTYPE,
  /* 10 */
  IFIN_TR69_ENCERR_PRIVALUE,             /*!< TODO  */
  IFIN_TR69_ENCERR_PARAMTYPE,
  IFIN_TR69_ENCERR_ADDRTYPE,
  IFIN_TR69_ENCERR_COMPOSITETYPE,
  IFIN_TR69_ENCERR_DISCRETETYPE,
  /* 15 */
  IFIN_TR69_ENCERR_DISPPARAMTYPE,          /*!< TODO  */
  IFIN_TR69_ENCERR_AINFOTYPE,
  IFIN_TR69_ENCERR_DIGRESPTYPE,
  IFIN_TR69_ENCERR_RESPTYPE,
  IFIN_TR69_ENCERR_STATUSTYPE,
  /* 20 */
  IFIN_TR69_ENCERR_BASICMETHOD,           /*!< TODO  */
  IFIN_TR69_ENCERR_URIPARAMTYPE,
  IFIN_TR69_ENCERR_TYPEOFHDR,
  IFIN_TR69_ENCERR_USERTYPE,
  IFIN_TR69_ENCERR_HOSTTYPE,
  /* 25 */
  IFIN_TR69_ENCERR_AUTHTYPE,              /*!< TODO  */
  IFIN_TR69_ENCERR_PATHTYPE,
  IFIN_TR69_ENCERR_PARTTYPE,
  IFIN_TR69_ENCERR_URITYPE,
  IFIN_TR69_ENCERR_SDPTYPE,
  /* 30 */
  IFIN_TR69_ENCERR_MSGTYPE,              /*!< TODO  */
  IFIN_TR69_ENCERR_MTYPE,
  IFIN_TR69_ENCERR_VIATYPE,
  IFIN_TR69_ENCERR_BOOL,
  IFIN_TR69_ENCERR_MEDIARANGE,
  /* 35 */
  IFIN_TR69_ENCERR_ACCEPTPARAM,           /*!< TODO  */
  IFIN_TR69_ENCERR_INFOTYPE,
  IFIN_TR69_ENCERR_DISPTYPE,
  IFIN_TR69_ENCERR_HANDLPARAMTYPE,
  IFIN_TR69_ENCERR_TELSUBTYPE,
  /* 40 */
  IFIN_TR69_ENCERR_PREFIXTYPE,             /*!< TODO  */
  IFIN_TR69_ENCERR_NWPREFIXTYPE,
  IFIN_TR69_ENCERR_AUTHRESP,
  IFIN_TR69_ENCERR_OUTOFMEM,
  IFIN_TR69_ENCINVLDHDRCOUNT,
  /* 45 */
  IFIN_TR69_ENCERRINVRPLPARAM,             /*!< TODO  */
  IFIN_TR69_ENCERR_INVEVENTPARAM,
  IFIN_TR69_ENCERR_INEVENTRSNVAL,
  IFIN_TR69_ENCERR_INSUBSTATEVAL,
  IFIN_TR69_ENCERR_INSUBEXPPARAM,
  /*50*/
  IFIN_TR69_SENT_491,                     /*!< TODO  */
  IFIN_TR69_RESPONSE_SENT,
  IFIN_TR69_DNS_ERROR,
  IFIN_TR69_QUERY_PENDING,

  /* 54 */
  IFIN_TR69_RTP_QUERY,
} e_IFIN_TR69_Ecode;



#endif

